// SimpleProtocolServerDLL.cpp : Defines the exported functions for the DLL application.
//

#define WASP_MAP_W_AND_A

#include "WASPPluginSDK\WASPDLLEntryPoints.h"

#include <string>

#pragma comment(lib, "ws2_32.lib")

using std::string;

// Per connection user data that is used to pass message framing state.

class CUserData
{
   public :

      CUserData()
         :  lastWasReturn(false)
      {

      }

      bool lastWasReturn;
};

// Message framing

unsigned long __cdecl GetMinimumMessageSize()
{
   return 2;
}

WaspDataLength __cdecl GetMessageSize(
   const WaspConnectionHandle /*connectionHandle*/,
   WaspConnectionUserData connectionUserData,
   const unsigned char * const pData,
   const WaspDataLength dataLength,
   const WaspDataLength previousLength)
{
   CUserData *pUserData = reinterpret_cast<CUserData *>(connectionUserData);

   for (WaspDataLength i = previousLength; i < dataLength; ++i)
   {
      if (pUserData->lastWasReturn)
      {
         pUserData->lastWasReturn = false;

         if (pData[i] == '\n')
         {
            // we actually want a count of characters, not a zero based
            // index, so we have to add 1...
            return i + 1; 
         }
      }
      else
      {
         if (pData[i] == '\r')
         {
            pUserData->lastWasReturn = true; 
         }
      }
   }

   return 0;
}

static WaspFunctions s_waspFunctions;

static void WriteString(
   const WaspConnectionHandle connectionHandle,
   const string &data)
{
   s_waspFunctions.writeToConnection(
      s_waspFunctions.id,
      connectionHandle,
      reinterpret_cast<const unsigned char *>(data.c_str()),
      static_cast<WaspDataLength>(data.length()));
}

// Message handling

WaspInstanceHandle __cdecl Initialise(
   WaspFunctions *pWaspFunctions,
   const char * /*pConfigValue*/)
{
   if (pWaspFunctions->size < sizeof(s_waspFunctions))
   {
      // By testing to see that the supplied function table is
      // larger than or equal to our function table we can 
      // return an error which indicates that we failed to
      // initialise if we have been built expecting a later 
      // (and therefore larger) function table than the version 
      // of WASP that is hosting us is supplying us.

      return 0;
   }

   memcpy(&s_waspFunctions, pWaspFunctions, sizeof(s_waspFunctions));

   s_waspFunctions.size = sizeof(s_waspFunctions);

   // return non zero for success...

   return 1;
}

WaspConnectionUserData __cdecl OnConnectionEstablished(
   const WaspInstanceHandle /*instanceHandle*/,
   const WaspConnectionHandle connectionHandle,
   const sockaddr * /*pAddress*/,
   const WaspAddressLength /*addressLength*/)
{
   WriteString(connectionHandle, "Welcome to Simple Protocol Server\r\n");

   return new CUserData();
}

void __cdecl OnReadCompleted(
   const WaspInstanceHandle /*instanceHandle*/,
   const WaspConnectionHandle connectionHandle,
   const WaspConnectionUserData connectionUserData,
   const unsigned char * const pDataIn,
   const WaspDataLength dataLength)
{
   string command;

   command.resize(dataLength);

   memcpy(const_cast<char *>(command.c_str()), pDataIn, dataLength);

   string echoCommand = command.substr(0, command.length() - 2); // strip the \r\n

   for (size_t i = 0; i < command.length(); ++i)
   {
      command[i] = (char)toupper(command[i]);
   }

   bool ok = false;

   if (command.length() >= 5)
   {
      // cant be a valid command, unless it's more than 5 chars      

      if (command.substr(0, 3) == "TOP")
      {
         ok = true;
      }
      else 
      {
         const std::string cmd = command.substr(0, 4);

         if (cmd == "APOP" ||
             cmd == "DELE" ||
             cmd == "LIST" ||
             cmd == "NOOP" ||
             cmd == "PASS" ||
             cmd == "QUIT" ||
             cmd == "RETR" ||
             cmd == "RSET" ||
             cmd == "STAT" ||
             cmd == "UIDL" ||
             cmd == "USER")
         {
            ok = true;
         }
      }
   }

   if (ok)
   {
      // We understand, but we aren't really a POP3 server...

      string response("-ERR sorry, we understand \"" +  echoCommand + "\", but, we're just a fake POP3 server...\r\n");

      WriteString(connectionHandle, response);
   }
   else
   {
      string response("-ERR sorry, we dont know what you mean, bye...\r\n");

      WriteString(connectionHandle, response);

      s_waspFunctions.shutdownConnection(s_waspFunctions.id, connectionHandle, WaspShutdownBoth);
   }
}

void __cdecl OnConnectionClientClosed(
   const WaspInstanceHandle /*instanceHandle*/,
   const WaspConnectionHandle connectionHandle,
   WaspConnectionUserData /*connectionUserData*/)
{
   // We don't expect the client to close first...

   s_waspFunctions.shutdownConnection(s_waspFunctions.id, connectionHandle, WaspShutdownBoth);
}

void __cdecl OnConnectionComplete(
   const WaspInstanceHandle /*instanceHandle*/,
   const WaspConnectionHandle /*connectionHandle*/,
   WaspConnectionUserData connectionUserData)
{
   CUserData *pUserData = reinterpret_cast<CUserData *>(connectionUserData);

   delete pUserData;
}
